@@ -1,16 +1,16 @@
-import re
-import math
 import datetime
-import requests
 import json
+import math
 import os
-import ssl
-import time
-import threading
 import queue
+import re
+import ssl
+import threading
+import time
+
+import requests
 from playsound import playsound
-from time import sleep
-import os
+
 if os.name == 'nt':
     import winsound
 
@@ -19,7 +19,7 @@ import pandas as pd
 from termcolor import cprint
 from inspect import currentframe
 
-__version__ = '0.1.27'
+__version__ = '0.1.30'
 
 __info__ = {
     '2022/01/04: 0.1.18 加入 __info__。',
@@ -30,19 +30,26 @@ __info__ = {
     '2022/01/11: 0.1.21 Add virtual function, check_place_cover() in ti class',
     '2022/02/10: 0.1.22 Add build_dealer_downloader()',
     '2022/02/14: 0.1.23 把 libexcel.py 搬到這邊來，以後可以全部沿用。',
-    '2022/06/13: 0.1.24 加入 tplaysound '
+    '2022/06/13: 0.1.24 加入 tplaysound ',
+    '2022/06/24: 0.1.28 加入 tLineNotify',
+    '2022/06/26: 0.1.29 加入 my_addressable_IP()',
+    '2022/06/29: 0.1.30 加入需要的module'
 }
 
 _MYPYLIB_ROOT = os.path.abspath(os.path.dirname(__file__))
 
-
-
 request_headers = {
     'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'
 }
 
 path_cronlog = 'cronlog'
 
+import urllib.request
+
+
+def my_addressable_IP():
+    return urllib.request.urlopen('https://ident.me').read().decode('utf8')
+
 
 def __LINE__():
     cf = currentframe()
@@ -597,6 +604,43 @@ def build_dealer_downloader(date=datetime.datetime(year=2020, month=1, day=1), t
             os.unlink(dealer_csv_path + '_')
 
 
+from line_notify import LineNotify
+
+
+class tLineNotify(threading.Thread):
+
+    def __init__(self, token, name=None):
+        threading.Thread.__init__(self)
+        self.line_notify = LineNotify(token, name=name)
+        self.queue = queue.Queue()
+        self.start()
+
+    def on(self):
+        self.line_notify.on()
+
+    def off(self):
+        self.line_notify.off()
+
+    def format(self, message):
+        return self.line_notify.format(message=message)
+
+    def send(self, message, image_path=None, sticker_id=None, package_id=None):
+        self.queue.put((message, image_path, sticker_id, package_id))
+
+    def run(self):
+        while True:
+            try:
+                (message, image_path, sticker_id, package_id) = self.queue.get(timeout=1)
+            except queue.Empty:
+                pass
+            else:
+                if message is None:
+                    break
+                self.line_notify.send(message=message, image_path=image_path, sticker_id=sticker_id, package_id=package_id)
+
+    def stop(self):
+        self.queue.put((None, None, None, None))
+
 
 class tplaysound(threading.Thread):
 
@@ -609,7 +653,6 @@ class tplaysound(threading.Thread):
 
         self.start()
 
-
     def play(self, filename=None):
         self.play_file(filename=filename)
 
@@ -643,8 +686,6 @@ class tplaysound(threading.Thread):
         self.queue.put(('stop', 'stop'))
 
 
-
-
 if __name__ == '__main__':
     from time import sleep
 
@@ -666,7 +707,6 @@ if __name__ == '__main__':
     if False:
         build_dealer_downloader()
 
-
     if False:
         all = load_all_shioaji_ticks()
         print(f'There are {len(all)} files')
@@ -699,4 +739,3 @@ if __name__ == '__main__':
         short_selling_to_csv()
 
         print(__LINE__())
-
